<?php
namespace Drupal\field_usage_tracker\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\views\Views;

class FieldUsageController extends ControllerBase {

  public function report() {
    $header = ['Field Name', 'Entity Type', 'Bundles', 'Used in Views?', 'Used in Code?'];
    $rows = [];

    $fields = FieldStorageConfig::loadMultiple();
    foreach ($fields as $field) {
      $entity_type = $field->getTargetEntityTypeId();
      $field_name = $field->getName();
      $bundles = implode(', ', array_keys($field->getBundles()));

      $used_in_views = $this->isFieldUsedInViews($field_name);
      $used_in_code = $this->isFieldUsedInCustomCode($field_name);

      $rows[] = [$field_name, $entity_type, $bundles, $used_in_views, $used_in_code];
    }

    $build['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No fields found.'),
    ];

    return $build;
  }

  protected function isFieldUsedInViews($field_name) {
	  $views = Views::getAllViews();
	  foreach ($views as $view) {
	    $view_display = $view->get('display');
	    foreach ($view_display as $display) {
	      if (isset($display['display_options'])) {
	        $display_options = $display['display_options'];

	        foreach (['fields', 'filters', 'sorts', 'relationships'] as $section) {
	          if (!empty($display_options[$section])) {
	            foreach ($display_options[$section] as $option) {
	              if (isset($option['field']) && $option['field'] === $field_name) {
	                return 'Yes';
	              }
	            }
	          }
	        }
	      }
	    }
	  }
	  return 'No';
	}

	protected function isFieldUsedInCustomCode($field_name) {
	  // Sanitize the field name to avoid injection.
	  $escaped_field = escapeshellarg($field_name);
	  $command = "grep -r --include='*.php' $escaped_field ../.. 2>/dev/null";

	  $output = shell_exec($command);
	  return !empty($output) ? 'Yes' : 'No';
	}

}
